/*	track.c
 *
 *	Experimental motion tracking.
 *
 *	Copyright 2000, Jeroen Vreeken
 *	This program is published under the GNU Public license
 */

#ifdef TRACKING

#include <termios.h>
#include "motion.h"
#include "conf.h"
#include "track.h"

extern struct config conf;

struct trackoptions track={
	0,	/*dev open*/
	NULL,	/*char *port*/
	0,	/*int motorx*/
	0,	/*int maxx;*/
};



static int stepper_command(int motor, int command, int n)
{
	char buffer[3];
	time_t timeout=time(NULL);

//	printf("Sending command 0x%02x 0x%02x 0x%02x\n", motor, command, n);
	buffer[0]=motor;
	buffer[1]=command;
	buffer[2]=n;
	if (write(track.dev, buffer, 3)!=3)
		return -1;

	while (read(track.dev, buffer, 1)!=1 && time(NULL) < timeout+1);
	if (time(NULL) >= timeout+2) {
		perror("Status byte timeout!");
		return 0;
	}
	return buffer[0];
}


static int stepper_status(int motor)
{
	return stepper_command(motor, COMMAND_STATUS, 0);
}


int track_center(void)
{
	struct termios adtio;

	if (!track.dev) {
		if ((track.dev=open(track.port, O_RDWR | O_NOCTTY)) < 0) {
			perror("Unable to open serial device");
			exit(1);
		}
		bzero (&adtio, sizeof(adtio));
		adtio.c_cflag= BAUDRATE | CS8 | CLOCAL | CREAD;
		adtio.c_iflag= IGNPAR;
		adtio.c_oflag= 0;
		adtio.c_lflag= 0;	/* non-canon, no echo */
		adtio.c_cc[VTIME]=0;	/* timer unused */
		adtio.c_cc[VMIN]=0;	/* blocking read until 1 char */
		tcflush (track.dev, TCIFLUSH);
		if (tcsetattr(track.dev, TCSANOW, &adtio) < 0) {
			perror("Unable to initialize serial device");
			exit(1);
		}
	}
	stepper_command(track.motorx, COMMAND_SPEED, 255);
	stepper_command(track.motorx, COMMAND_LEFT_N, track.maxx);
	while (!(stepper_status(track.motorx) & STATUS_SAFETYL));
	stepper_command(track.motorx, COMMAND_RIGHT_N, track.maxx/2);
	while (stepper_status(track.motorx) & STATUS_RIGHT);

	return 0;
}

int track_move(int dev, int pipe, int mpipe, char *out, char *new, char *ref, int width, int height)
{
	struct coord center;
	int command=0;
	int n=0;
	
	center=locate_diffs_center(out, width, height);
	if (center.x < width/2) {
		command=COMMAND_LEFT_N;
		n=width/2-center.x;
	}
	if (center.x > width/2) {
		command=COMMAND_RIGHT_N;
		n=center.x-width/2;
	}
	n=n*40/width;
	if (n) {
		stepper_command(track.motorx, command, n);
		while(stepper_status(track.motorx) & (STATUS_RIGHT|STATUS_LEFT) &&
		    next_capture(dev, new, width, height)) {
			for (n=0; n<width*height*3; n++)
				ref[n]=new[n];
			if (conf.vidpipe)
				put_videopipe(pipe, new, width, height);
			if (conf.motionvidpipe)
				put_videopipe(mpipe, out, width, height);
		}
		return 1;
	}
	return 0;
}
#endif /* TRACKING */
